#!/usr/bin/env python
# -*- coding: utf-8 -*-


# ==============================================================================
#
#       File Name : quenv/migrations/0001_initial.py
#
#       Creation Date : Fri 31 Jul 2020 10:51:23 PM EEST (22:51)
#
#       Last Modified :
#
# ==============================================================================

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Date",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("check_date", models.DateField(verbose_name="Check Date")),
            ],
            options={
                "verbose_name": "Check Date",
                "verbose_name_plural": "Check Dates",
            },
        ),
        migrations.CreateModel(
            name="Environment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "environment_name",
                    models.CharField(
                        max_length=31, unique=True, verbose_name="Environment Name"
                    ),
                ),
            ],
            options={
                "verbose_name": "Environment",
                "verbose_name_plural": "Environments",
            },
        ),
        migrations.CreateModel(
            name="Grade",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "level",
                    models.CharField(max_length=3, unique=True, verbose_name="Level"),
                ),
                (
                    "ordering",
                    models.PositiveSmallIntegerField(blank=True, verbose_name="Order"),
                ),
            ],
            options={"verbose_name": "Grade", "verbose_name_plural": "Grades"},
        ),
        migrations.CreateModel(
            name="License",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "license_name",
                    models.CharField(
                        max_length=63, unique=True, verbose_name="License Name"
                    ),
                ),
            ],
            options={"verbose_name": "License", "verbose_name_plural": "Licenses"},
        ),
        migrations.CreateModel(
            name="Package",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "package_name",
                    models.CharField(
                        max_length=31, unique=True, verbose_name="Package Name"
                    ),
                ),
            ],
            options={"verbose_name": "Package", "verbose_name_plural": "Packages"},
        ),
        migrations.CreateModel(
            name="Info",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("lines", models.PositiveIntegerField(verbose_name="Lines of Code")),
                (
                    "date_key",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="quenv.Date",
                        verbose_name="Date Checked",
                    ),
                ),
                (
                    "environment_key",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="quenv.Environment",
                        verbose_name="Environment",
                    ),
                ),
                (
                    "grade_key",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="quenv.Grade",
                        verbose_name="Grade",
                    ),
                ),
                (
                    "license_description",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="quenv.License",
                        verbose_name="License Description",
                    ),
                ),
                (
                    "licenses_keys",
                    models.ManyToManyField(
                        related_name="packages",
                        to="quenv.License",
                        verbose_name="Licenses",
                    ),
                ),
                (
                    "package_key",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="quenv.Package",
                        verbose_name="Package",
                    ),
                ),
            ],
            options={
                "verbose_name": "Installation Info",
                "verbose_name_plural": "Installation Info",
            },
        ),
        migrations.CreateModel(
            name="IncrementalChanges",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "license_changed",
                    models.BooleanField(verbose_name="Changed License"),
                ),
                ("added", models.BooleanField(verbose_name="Added")),
                ("removed", models.BooleanField(verbose_name="Removed")),
                (
                    "date_key",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="quenv.Date",
                        verbose_name="Date Checked",
                    ),
                ),
                (
                    "package_key",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="quenv.Package",
                        verbose_name="Package",
                    ),
                ),
            ],
            options={
                "verbose_name": "Incremental Changes",
                "verbose_name_plural": "Incremental Changes",
            },
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-


# ==============================================================================
#
#       File Name : quenv/managers.py
#
#       Creation Date : Wed 24 Apr 2019 10:33:22 PM EEST (22:33)
#
#       Last Modified : Mon 06 May 2019 08:48:38 PM EEST (20:48)
#
# ==============================================================================


from django.db import models


class LicenseManager(models.Manager):
    def get_by_natural_key(self, license_name):
        return self.get(license_name=license_name)


class PackageManager(models.Manager):
    def get_by_natural_key(self, package_name):
        return self.get(package_name=package_name)


class EnvironmentManager(models.Manager):
    def get_by_natural_key(self, environment_name):
        return self.get(environment_name=environment_name)


class GradeManager(models.Manager):
    def get_by_natural_key(self, level):
        return self.get(level=level)


class DateManager(models.Manager):
    def get_by_natural_key(self, check_date):
        return self.get(check_date=check_date)


class IncrementalChangesManager(models.Manager):
    def get_by_natural_key(self, package_key, date_key):
        return self.get(package_key=package_key, date_key=date_key)


class InfoManager(models.Manager):
    def get_by_natural_key(self, date, environment, package):
        return self.get(
            date_key__check_date=date,
            environment_key__environment_name=environment,
            package_key__package_name=package,
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-


# ==============================================================================
#
#       File Name : quenv/migrations/0002_auto_20200731_1947.py
#
#       Creation Date : Fri 31 Jul 2020 10:51:23 PM EEST (22:51)
#
#       Last Modified :
#
# ==============================================================================

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("quenv", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="environment",
            name="environment_name",
            field=models.CharField(
                max_length=63, unique=True, verbose_name="Environment Name"
            ),
        ),
        migrations.AlterField(
            model_name="license",
            name="license_name",
            field=models.CharField(
                max_length=127, unique=True, verbose_name="License Name"
            ),
        ),
        migrations.AlterField(
            model_name="package",
            name="package_name",
            field=models.CharField(
                max_length=63, unique=True, verbose_name="Package Name"
            ),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-


# ==============================================================================
#
#       File Name : quenv/admin.py
#
#       Creation Date : Sat 20 Apr 2019 08:39:37 PM EEST (20:39)
#
#       Last Modified : Tue 07 May 2019 03:52:11 PM EEST (15:52)
#
# ==============================================================================

from django.contrib import admin
from django.db.models import Q
from django.urls import reverse
from django.utils.html import format_html
from gettext import gettext as _
from quenv import models


class LicenseListFilter(admin.SimpleListFilter):
    title = _("License in Description or Classifier")
    parameter_name = "license"

    def lookups(self, request, model_admin):
        licenses = models.License.objects.all().order_by("license_name")
        for each in licenses:
            yield (each.license_name, each.license_name)

    def queryset(self, request, queryset):

        if self.value():
            search_terms = Q(
                Q(license_description__license_name=self.value())
                | Q(licenses_keys__license_name=self.value())
            )
            return queryset.filter(search_terms).distinct()


@admin.register(models.Info)
class InfoAdmin(admin.ModelAdmin):
    list_filter = ("environment_key__environment_name", LicenseListFilter)
    list_select_related = True
    date_hierarchy = "date_key__check_date"
    list_display = (
        "packages",
        "licenses_link",
        "lines",
        "grade_key_level",
        "changes_link",
    )

    @staticmethod
    def packages(obj):
        return obj.package_key.package_name

    packages.admin_order_field = "package_key__package_name"

    @staticmethod
    def licenses_link(obj):
        url = reverse("admin:quenv_license_changelist")
        return format_html(
            "<a href={0}?packages__id={1}>{2}</a>", url, obj.id, obj.license_description
        )

    licenses_link.short_description = _("License Description")
    licenses_link.admin_order_field = "license_description__license_name"

    @staticmethod
    def grade_key_level(obj):
        return obj.grade_key.level

    grade_key_level.admin_order_field = "grade_key__ordering"

    @staticmethod
    def changes_link(obj):
        url = reverse("admin:quenv_incrementalchanges_changelist")
        return format_html(
            "<a href={0}?date_key__check_date__day={1}&date_key__check_date__month={2}&date_key__check_date__year={3}>Related Changes</a>",
            url,
            obj.date_key.check_date.day,
            obj.date_key.check_date.month,
            obj.date_key.check_date.year,
        )

    changes_link.short_description = _("Related Changes")

    @staticmethod
    def has_add_permission(request):
        return False

    @staticmethod
    def has_change_permission(request, obj=None):
        return False

    @staticmethod
    def has_delete_permission(request, obj=None):
        return False


@admin.register(models.License)
class LicenseAdmin(admin.ModelAdmin):
    list_display = ("license_name", "packages_link")

    @staticmethod
    def packages_link(obj):
        url = reverse("admin:quenv_info_changelist")
        return format_html(
            "<a href={0}?license={1}>Related Packages</a>",
            url,
            obj.license_name.replace(" ", "+"),
        )

    packages_link.short_description = _("Related Packages")

    @staticmethod
    def has_add_permission(request):
        return False

    @staticmethod
    def has_change_permission(request, obj=None):
        return False

    @staticmethod
    def has_delete_permission(request, obj=None):
        return False


@admin.register(models.Package)
class PackageAdmin(admin.ModelAdmin):
    list_display = ("package_name",)

    @staticmethod
    def has_add_permission(request):
        return False

    @staticmethod
    def has_change_permission(request, obj=None):
        return False

    @staticmethod
    def has_delete_permission(request, obj=None):
        return False


@admin.register(models.Environment)
class EnvironmentAdmin(admin.ModelAdmin):
    @staticmethod
    def has_add_permission(request):
        return False

    @staticmethod
    def has_change_permission(request, obj=None):
        return False

    @staticmethod
    def has_delete_permission(request, obj=None):
        return False


@admin.register(models.Date)
class DateAdmin(admin.ModelAdmin):
    list_display = ("check_date", "packages_link", "changes_link")

    @staticmethod
    def packages_link(obj):
        url = reverse("admin:quenv_info_changelist")
        return format_html(
            "<a href={0}?date_key__check_date__day={1}&date_key__check_date__month={2}&date_key__check_date__year={3}>Related Packages</a>",
            url,
            obj.check_date.day,
            obj.check_date.month,
            obj.check_date.year,
        )

    packages_link.short_description = _("Related Packages")

    @staticmethod
    def changes_link(obj):
        url = reverse("admin:quenv_incrementalchanges_changelist")
        return format_html(
            "<a href={0}?date_key__check_date__day={1}&date_key__check_date__month={2}&date_key__check_date__year={3}>Related Changes</a>",
            url,
            obj.check_date.day,
            obj.check_date.month,
            obj.check_date.year,
        )

    changes_link.short_description = _("Related Changes")

    @staticmethod
    def has_add_permission(request):
        return False

    @staticmethod
    def has_change_permission(request, obj=None):
        return False

    @staticmethod
    def has_delete_permission(request, obj=None):
        return False


@admin.register(models.Grade)
class GradeAdmin(admin.ModelAdmin):
    ordering = ["ordering"]

    @staticmethod
    def has_add_permission(request):
        return False

    @staticmethod
    def has_change_permission(request, obj=None):
        return False

    @staticmethod
    def has_delete_permission(request, obj=None):
        return False


@admin.register(models.IncrementalChanges)
class IncrementalChangesAdmin(admin.ModelAdmin):
    date_hierarchy = "date_key__check_date"
    list_display = (
        "package_key",
        "license_changed",
        "added",
        "removed",
        "packages_link",
    )

    @staticmethod
    def packages_link(obj):
        url = reverse("admin:quenv_info_changelist")
        return format_html(
            "<a href={0}?date_key__check_date__day={1}&date_key__check_date__month={2}&date_key__check_date__year={3}>Related Installation</a>",
            url,
            obj.date_key.check_date.day,
            obj.date_key.check_date.month,
            obj.date_key.check_date.year,
        )

    packages_link.short_description = _("Installation Packages")

    @staticmethod
    def has_add_permission(request):
        return False

    @staticmethod
    def has_change_permission(request, obj=None):
        return False

    @staticmethod
    def has_delete_permission(request, obj=None):
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-


# ==============================================================================
#
#       File Name : setup.py
#
#       Creation Date : Mon 08 Apr 2019 07:00:40 PM EEST (19:00)
#
#       Last Modified : Fri 31 Jul 2020 10:52:15 PM EEST (22:52)
#
# ==============================================================================

import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="django-quenv",
    version="0.1.5",
    python_requires=">=3.5",
    description=(
        "A django application that gathers the names of all installed packages "
        "in a virtualenv, their licenses along with related quality metrics, "
        "in django-admin."
    ),
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/raratiru/django-quenv",
    author="George Tantiras",
    license="BSD 3-Clause License",
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    install_requires=["setuptools", "tqdm", "Django", "requests"],
    zip_safe=False,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Natural Language :: English",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Internet :: WWW/HTTP :: Site Management",
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


# ==============================================================================
#
#       File Name : quenv/models.py
#
#       Creation Date : Sat 20 Apr 2019 08:39:37 PM EEST (20:39)
#
#       Last Modified : Fri 31 Jul 2020 10:47:11 PM EEST (22:47)
#
# ==============================================================================

from django.db import models
from gettext import gettext as _
from quenv import managers


class License(models.Model):
    license_name = models.CharField(
        verbose_name=_("License Name"), max_length=127, unique=True
    )

    objects = managers.LicenseManager()

    def __str__(self):
        return "{0}".format(self.license_name)

    def natural_key(self):
        return (self.license_name,)

    class Meta:
        verbose_name = _("License")
        verbose_name_plural = _("Licenses")


class Package(models.Model):
    package_name = models.CharField(
        verbose_name=_("Package Name"), max_length=63, unique=True
    )

    objects = managers.PackageManager()

    def __str__(self):
        return "{0}".format(self.package_name)

    def natural_key(self):
        return (self.package_name,)

    class Meta:
        verbose_name = _("Package")
        verbose_name_plural = _("Packages")


class Environment(models.Model):
    environment_name = models.CharField(
        verbose_name=_("Environment Name"), max_length=63, unique=True
    )

    objects = managers.EnvironmentManager()

    def __str__(self):
        return "{0}".format(self.environment_name)

    def natural_key(self):
        return (self.environment_name,)

    class Meta:
        verbose_name = _("Environment")
        verbose_name_plural = _("Environments")


class Grade(models.Model):
    level = models.CharField(verbose_name=_("Level"), max_length=3, unique=True)
    ordering = models.PositiveSmallIntegerField(verbose_name=_("Order"), blank=True)

    objects = managers.GradeManager()

    def __str__(self):
        return "{0}".format(self.level)

    def natural_key(self):
        return (self.level,)

    class Meta:
        verbose_name = _("Grade")
        verbose_name_plural = _("Grades")


class Date(models.Model):
    check_date = models.DateField(verbose_name=_("Check Date"))

    objects = managers.DateManager()

    def natural_key(self):
        return (self.check_date.strftime("%Y-%m-%d"),)

    def __str__(self):
        return "{0}".format(self.check_date.strftime("%Y-%m-%d"))

    class Meta:
        verbose_name = _("Check Date")
        verbose_name_plural = _("Check Dates")


class Info(models.Model):
    environment_key = models.ForeignKey(
        Environment, verbose_name=_("Environment"), on_delete=models.PROTECT
    )
    package_key = models.ForeignKey(
        Package, verbose_name=_("Package"), on_delete=models.PROTECT
    )
    license_description = models.ForeignKey(
        License, verbose_name=_("License Description"), on_delete=models.PROTECT
    )
    lines = models.PositiveIntegerField(verbose_name=_("Lines of Code"))
    grade_key = models.ForeignKey(
        Grade, verbose_name=_("Grade"), on_delete=models.PROTECT
    )
    licenses_keys = models.ManyToManyField(
        License, verbose_name=_("Licenses"), related_name="packages"
    )
    date_key = models.ForeignKey(
        Date, verbose_name=_("Date Checked"), on_delete=models.PROTECT
    )

    objects = managers.InfoManager()

    def __str__(self):
        return "Installation id: {0}".format(self.id)

    def natural_key(self):
        return (
            tuple()
            + self.date_key.natural_key()
            + self.environment_key.natural_key()
            + self.package_key.natural_key()
        )

    natural_key.dependencies = ["quenv.environment", "quenv.package", "quenv.date"]

    class Meta:
        verbose_name = _("Installation Info")
        verbose_name_plural = _("Installation Info")


class IncrementalChanges(models.Model):
    package_key = models.ForeignKey(
        Package, verbose_name=_("Package"), on_delete=models.PROTECT
    )
    license_changed = models.BooleanField(verbose_name=_("Changed License"))
    added = models.BooleanField(verbose_name=_("Added"))
    removed = models.BooleanField(verbose_name=_("Removed"))
    date_key = models.ForeignKey(
        Date, verbose_name=_("Date Checked"), on_delete=models.PROTECT
    )

    def __str__(self):
        return "Change id: {0}".format(self.id)

    def natural_key(self):
        return (
            (self.date,) + self.package_key.natural_key() + self.date_key.natural_key()
        )

    natural_key.dependencies = ["quenv.package", "quenv.date"]

    class Meta:
        verbose_name = _("Incremental Changes")
        verbose_name_plural = _("Incremental Changes")



#!/usr/bin/env python
# -*- coding: utf-8 -*-


# ==============================================================================
#
#       File Name : quenv/licenses.py
#
#       Creation Date : Sun 21 Apr 2019 01:18:22 PM EEST (13:18)
#
#       Last Modified : Tue 07 May 2019 04:00:36 PM EEST (16:00)
#
# ==============================================================================

import json
import os
import requests
import sys
import time

from collections import namedtuple, defaultdict
from datetime import date
from django.conf import settings
from django.core.management import call_command
from itertools import compress
from pkg_resources import get_distribution, working_set
from quenv.models import Date, Info, IncrementalChanges
from tqdm import tqdm
from urllib.parse import urlparse


class Package:
    """
    It creates a pkg_resources.DistInfoDistribution (setuptools) object
    and extracts the necessary information from its metadata attributes
    and classifiers.
    """

    def __init__(self, package_name):
        self.pkg = get_distribution(package_name)
        self.result = namedtuple(
            "Result",
            [
                "name",
                "environment",
                "description_license",
                "classifier_licenses",
                "lgtm",
            ],
        )
        # self.result.name = '{0}-{1}'.format(self.pkg.project_name, self.pkg.version)
        self.result.name = "{0}".format(self.pkg.project_name)
        self.result.environment = sys.executable.split(os.sep)[-3]
        self.service_path = "https://lgtm.com/api/v1.0/projects/"

    def get_lgtm_result(self, url_identifier):
        """
        Query the lgtm api and get the available data if they exist.
        """
        if url_identifier:
            full_path = "{0}{1}".format(self.service_path, url_identifier)

            time.sleep(1)  # Taking it easier
            response = requests.get(full_path, headers={"Accept": "application/json"})

            if response.status_code == 200:
                for each_lang in json.loads(response.text).get("languages", []):
                    if each_lang.get("language", None) == "python":
                        return each_lang
        return {}

    def lgtm_query(self, line):
        """
        Check if a github url is present in order to query the lgtm api.
        It is possible to extend the reasons for starting a lgtm query.
        """
        try:
            url = urlparse(line[line.find("https://") :])
        except ValueError:
            return self.get_lgtm_result(None)

        whitelist = ("github.com",)
        if all(
            (
                any(
                    (
                        line.startswith("Project-URL: Source"),
                        line.startswith("Home-page"),
                    )
                ),
                url.netloc in whitelist,
            )
        ):
            return self.get_lgtm_result("g{0}".format(url.path))

        return self.get_lgtm_result(None)

    def filters(self, line):
        """
        Search in each metadata line for package licenses
        or reasons (like the string 'github.com') to start a lgtm query.
        """
        return {
            "lgtm": self.lgtm_query(line),
            "licenses": compress(
                (line[9:], line[23:].replace("OSI Approved", "").replace(" :: ", "")),
                (line.startswith("License:"), line.startswith("Classifier: License")),
            ),
        }

    def get_pkg_details(self):
        try:
            lines = self.pkg.get_metadata_lines("METADATA")
        except (OSError, KeyError):
            lines = self.pkg.get_metadata_lines("PKG-INFO")
        return map(self.filters, lines)

    def info(self):
        lgtm = dict()
        classifiers = list()

        for each in self.get_pkg_details():
            lgtm.update(each["lgtm"])
            classifiers.extend(filter(None, each["licenses"]))

        try:
            self.result.description_license = classifiers.pop(0)
        except IndexError:
            pass
        self.result.classifier_licenses = classifiers
        self.result.lgtm = lgtm
        return self.result


class Dump:
    """
    It creates a json dump based on the json building blocks for each model.
    It also provides default data for Grade and License models.
    """

    def __init__(self):
        self.raw_data = defaultdict(set)
        self.data = defaultdict(list)
        self.environment_packages = working_set
        self.ordering = dict(
            [
                (grade, index)
                for index, grade in enumerate(("A+", "A", "B", "C", "D", "E", "0"))
            ]
        )
        self.create_package = Package
        for k, v in self.ordering.items():
            self.raw_data[self.get_grade_obj].add((k, v))
        self.raw_data[self.get_license_obj].add("UNKNOWN")

    @staticmethod
    def get_environment_obj(environment):
        return {
            "model": "quenv.environment",
            "fields": {"environment_name": str(environment).strip()},
        }

    @staticmethod
    def get_license_obj(license_name):
        return {
            "model": "quenv.license",
            "fields": {"license_name": str(license_name).strip()},
        }

    @staticmethod
    def get_package_obj(package):
        return {
            "model": "quenv.package",
            "fields": {"package_name": str(package).strip()},
        }

    @staticmethod
    def get_date_obj():
        return {
            "model": "quenv.date",
            "fields": {"check_date": date.today().strftime("%Y-%m-%d")},
        }

    @staticmethod
    def get_grade_obj(grade, ordering=10000):
        return {
            "model": "quenv.grade",
            "fields": {"level": str(grade).strip(), "ordering": int(ordering)},
        }

    @staticmethod
    def get_info_obj(
        environment, package, description="UNKNOWN", lines=0, grade="0", licenses=None
    ):
        return {
            "model": "quenv.info",
            "fields": {
                "environment_key": [str(environment).strip()],
                "package_key": [str(package).strip()],
                "license_description": [str(description).strip()],
                "lines": int(lines),
                "grade_key": [str(grade).strip()],
                "date_key": [date.today().strftime("%Y-%m-%d")],
                "licenses_keys": [[str(l.strip())] for l in licenses]
                if licenses
                else [],
            },
        }

    def compose_data(self, package_info):
        """
        Use raw_data dict to compose the contents of the data instance variable.
        """
        self.data[self.get_date_obj].append(self.get_date_obj())

        for function, values in self.raw_data.items():
            if function == self.get_grade_obj or function == self.get_info_obj:
                self.data[function].extend(list(map(lambda x: function(*x), values)))
            else:
                self.data[function].extend(list(map(function, values)))

    def prepare_raw_data(self, package_info):
        """
        From each package info get its gathered data and distribute them to
        the raw_data instance variable in order to remove possible duplicates.
        """
        self.raw_data[self.get_license_obj].add(package_info.description_license)
        self.raw_data[self.get_package_obj].add(package_info.name)
        self.raw_data[self.get_environment_obj].add(package_info.environment)
        self.raw_data[self.get_info_obj].add(
            (
                package_info.environment,
                package_info.name,
                package_info.description_license,
                package_info.lgtm.get("lines", 0),
                package_info.lgtm.get("grade", 0),
                tuple(package_info.classifier_licenses),
            )
        )

        for each_license in package_info.classifier_licenses:
            self.raw_data[self.get_license_obj].add(each_license)

        if package_info.lgtm.get("grade", False):
            self.raw_data[self.get_grade_obj].add(
                (
                    package_info.lgtm["grade"],
                    self.ordering.get(package_info.lgtm["grade"], 10000),
                )
            )

    def get_result(self, env_packages):
        with tqdm(total=len(tuple(self.environment_packages))) as pbar:
            for each_package in env_packages:
                package_info = each_package.info()
                self.prepare_raw_data(package_info)
                pbar.update(1)
            self.compose_data(package_info)
        return (
            self.data[self.get_environment_obj]
            + self.data[self.get_license_obj]
            + self.data[self.get_date_obj]
            + self.data[self.get_package_obj]
            + self.data[self.get_grade_obj]
            + self.data[self.get_info_obj]
        )

    @staticmethod
    def update_db(fixture_fullpath):
        call_command("loaddata", fixture_fullpath)
        dates = Date.objects.all().order_by("-check_date")[:2]
        difference_collection = Info.objects.filter(date_key__in=dates).values_list(
            "date_key__check_date",
            "package_key_id",
            "license_description",
            "licenses_keys",
        )
        packages = defaultdict(set)
        licenses = dict()

        for each in difference_collection:
            packages[each[0]].add(each[1])
            licenses.setdefault(each[1], {}).update({each[0]: (each[2], each[3])})
        changed_licenses = []

        for k, v in licenses.items():
            if len(v) == 2:
                if v[dates[0].check_date] != v[dates[1].check_date]:
                    changed_licenses.append(k)

        if len(dates) == 2:
            if not IncrementalChanges.objects.filter(date_key=dates[0]).exists():
                IncrementalChanges.objects.bulk_create(
                    [
                        IncrementalChanges(
                            package_key_id=i,
                            license_changed=False,
                            added=True,
                            removed=False,
                            date_key=dates[0],
                        )
                        for i in (
                            packages[dates[0].check_date].difference(
                                packages[dates[1].check_date]
                            )
                        )
                    ]
                )
                IncrementalChanges.objects.bulk_create(
                    [
                        IncrementalChanges(
                            package_key_id=i,
                            license_changed=False,
                            added=False,
                            removed=True,
                            date_key=dates[0],
                        )
                        for i in (
                            packages[dates[1].check_date].difference(
                                packages[dates[0].check_date]
                            )
                        )
                    ]
                )
                IncrementalChanges.objects.bulk_create(
                    [
                        IncrementalChanges(
                            package_key_id=i,
                            license_changed=True,
                            added=False,
                            removed=False,
                            date_key=dates[0],
                        )
                        for i in changed_licenses
                    ]
                )

    def dump(self):
        fixture_fullpath = os.path.join(
            getattr(settings, "QUENV_PATH", ""),
            "{0}.json".format(date.today().strftime("%Y-%m-%d")),
        )
        data = self.get_result(map(self.create_package, self.environment_packages))
        with open(fixture_fullpath, "x") as f:
            json.dump(data, f, indent=2)

        if getattr(settings, "QUENV_UPDATE_DB", True):
            self.update_db(fixture_fullpath)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


# ==============================================================================
#
#       File Name : quenv/management/commands/quenv.py
#
#       Creation Date : Mon 06 May 2019 09:01:59 PM EEST (21:01)
#
#       Last Modified : Tue 07 May 2019 12:13:29 PM EEST (12:13)
#
# ==============================================================================

from django.core.management.base import BaseCommand, CommandError
from gettext import gettext as _
from quenv.licenses import Dump


class Command(BaseCommand):
    help = _(
        "Scans the active python environment for installed packages, reports their licenses and quality"
    )

    def handle(self, *args, **options):
        obj = Dump()
        try:
            obj.dump()
        except FileExistsError:
            raise CommandError(
                _(
                    "Quenv operates only once per day. A fixture for today already exists."
                )
            )

        self.stdout.write(
            self.style.SUCCESS(_("Successfully created all quenv fixtures"))
        )
